#pragma once
#include "ui_CuttingStockApp.h"
#include "GraphicsStruct.h"
#include "PolygonView.h"
#include <xml/xmlParser.h>

using namespace cutting_stock_graphics;

namespace cutting_stock
{
	class CuttingStockApp : public QMainWindow
	{
		Q_OBJECT

	public:
		explicit CuttingStockApp(QWidget *parent = nullptr);
		~CuttingStockApp();

		void init();
		void setWindowTitleFromFile(const std::string&);
		static int getArgs(char*& arg0, char*& arg1);

		PolygonView* colours;
		QLabel* statusMessage;
		std::vector<GraphicsStructP> graphicsVector;
		std::string xmlFile;
		std::string inputFile;
		int currentIndex = 0;
		int dimension = 0;
		int iterations = 1;
		static const char* TITLE;
		static const char* PARAMS;
		static const char* INPUT_PATH;

	public slots:
		void getSolution(const QVector<cutting_stock_graphics::GraphicsStructP> &graphicsVector);
		void setStatusText(const QString&) const;
		void setStatusMessageFromFitness(const double&) const;

	private:
		Ui::CuttingStockAppClass ui;
		void drawInitialSolution();
		void drawSolution();
		void setFitness();
		void parseXml(std::string);
		void updateXml() const;
		static void setTextFieldFromNode(QLineEdit* textField, const XMLNode& parent, const char* attribute);
		static void updateXmlNodeFromTextField(QLineEdit* textField, const XMLNode& parent, const char* attribute);

	private slots:
		void openFile();
		void openOutput();
		void prevSolution();
		void nextSolution();
		void startExecute();
		// errorString(QString)
	};
}