#include "BlfWorker.h"
#include "GenerationOp.h"
#include "CuttingStockEvalOp.h"
#include "CuttingStockApp.h"
#include "cuttingAlgorithm.h"

using std::vector;
using std::string;
using namespace cutting_stock_graphics;

namespace cutting_stock
{
	BlfWorker::BlfWorker(int iterations, string infile) {
		this->iterations = iterations;
		this->infile = infile;
	}

	BlfWorker::~BlfWorker() {
	}

	void BlfWorker::getGeneration(QString text) {
		emit sendGeneration(text);
	}

	void BlfWorker::process() {
		char *arg0, *arg1;
		const int argc = CuttingStockApp::getArgs(arg0, arg1);
		char* argv[] = { arg0, arg1, nullptr };

		// otvori izlaznu datoteku
		const auto outputFile = "output/" + infile + ".out";
		ofstream fout(outputFile);
		if (!fout.is_open())
			throw runtime_error("Cannot open output file.");

		auto genOpP = boost::make_shared<GenerationOp>();
		connect(genOpP.get(), SIGNAL(setGeneration(QString)), this, SLOT(getGeneration(QString)));
		const CuttingStockEvalOpP tsp(new CuttingStockEvalOp);

		vector<GraphicsStructP> graphicsVector;
		for (auto i = 0; i < iterations; ++i) {
			StateP state(new State);
			state->setEvalOp(tsp);
			state->addOperator(genOpP);
			state->initialize(argc, argv);
			state->run();

			const auto genotype = state->getHoF()->getBest()[0]->getGenotype();
			auto permutation = boost::static_pointer_cast<Permutation::Permutation> (genotype);

			// zapisi rjesenje u datoteku
			const auto size = permutation->variables.size();
			for (auto j = 0; j < size; ++j) {
				fout << permutation->variables[j] << '\t';
			}

			const GraphicsStructP solution = getBlfSolution(tsp, permutation->variables);
			graphicsVector.push_back(solution);
			fout << endl;
		}

		fout.close();
		delete[] arg0;
		delete[] arg1;

		emit setSolution(QVector<GraphicsStructP>::fromStdVector(graphicsVector));
		emit finished();
	}
}