#include "PolygonView.h"

using std::vector;

namespace cutting_stock_graphics
{
	PolygonView::PolygonView(QWidget *parent)
		: QWidget(parent), canvasHeight(0), width(0), ymax(0)
	{
	}

	void PolygonView::paintEvent(QPaintEvent *e) {
		Q_UNUSED(e);

		doPainting();
	}

	void PolygonView::doPainting() {
		QPainter painter(this);
		painter.setPen(QColor("#d4d4d4"));

		// set colors
		vector<QBrush> brushes;		
		brushes.push_back(QBrush("#1ac500"));
		brushes.push_back(QBrush("#539e47"));
		brushes.push_back(QBrush("#004fc5"));
		brushes.push_back(QBrush("#c50024"));

		brushes.push_back(QBrush("#9e4757"));
		brushes.push_back(QBrush("#5f3b00"));
		brushes.push_back(QBrush("#4c4c4c"));
		brushes.push_back(QBrush("#785f36"));

		brushes.push_back(QBrush("#ff4b84"));
		brushes.push_back(QBrush("#a349a4"));
		brushes.push_back(QBrush("#ff7f27"));
		brushes.push_back(QBrush("#d5b72b"));

		brushes.push_back(QBrush("#00a2e8"));
		brushes.push_back(QBrush("#7092be"));

		brushes.push_back(QBrush("#c56c00"));

		canvasHeight = parentWidget()->height();
		const auto h = static_cast<float>(canvasHeight);
		const auto w = static_cast<float>(parentWidget()->width());
		const auto hScale = w / width;
		const auto vScale = h / ymax;
		const auto scale = hScale < vScale ? hScale : vScale;

		const auto size = elements.size();
		for (auto i = 0; i < size; ++i) {
			auto p = elements[i];
			painter.setBrush(brushes[(p.number - 1) % 14]);
			painter.drawRect(scale * p.coordinates[3].x, (canvasHeight - scale * p.coordinates[3].y), scale * p.width, scale * p.height);
		}

		painter.setBrush(brushes[14]);
		painter.drawLine(scale * width, 0, scale * width, scale * ymax);
	}
}