#include "cuttingAlgorithm.h"

using std::vector;
using std::set;

namespace cutting_stock
{
	double blf(CuttingStockEvalOpP tsp, const vector<int> &elements)
	{
		const auto solution = getBlfSolution(tsp, elements);
		return solution->fitness;
	}

	GraphicsStructP getBlfSolution(CuttingStockEvalOpP tsp, const vector<int>& elements)
	{
		// lista smjestenih elemenata
		vector<Polygon> placedElements;
		// lista slobodnih tocaka za smjestanje poligona (sortirano po y pa x)
		set<Polygon::Point, Polygon::PointCompare> insertPoints;
		insertPoints.insert(Polygon::Point{ 0, 0 });

		const auto& size = elements.size();
		uint ypos = 0, ymax = 0;
		for (auto i = 0; i < size; ++i) {
			auto p = tsp->elements[elements[i]];
			boost::scoped_ptr<Polygon::Point> chosenPoint;

			foreach(const Polygon::Point point, insertPoints) {
				if (!tsp->canBePlaced(p, point, placedElements)) continue;
				chosenPoint.reset(new Polygon::Point);
				*chosenPoint = point;
				break;
			}

			if (chosenPoint != nullptr) {
				insertPoints.erase(*chosenPoint);
			} else {
				if (p.width > tsp->width) {
					tsp->totalArea = 0;
					break;
				}
				chosenPoint.reset(new Polygon::Point);
				*chosenPoint = Polygon::Point{ 0, int(ymax) };
			}
			insertPoints.insert(Polygon::Point{ chosenPoint->x + p.width, chosenPoint->y });
			insertPoints.insert(Polygon::Point{ chosenPoint->x, chosenPoint->y + p.height });
			p.adjust(chosenPoint->x, chosenPoint->y);

			ypos = p.coordinates[0].y + p.height; // (yk + hk)
			if (ypos > ymax) ymax = ypos;
			placedElements.push_back(p);
		}

		const double fitnessV =  int(double(tsp->totalArea) / (tsp->width * ymax) * 100);
		const auto p(boost::make_shared<GraphicsStruct>(
			placedElements, fitnessV, tsp->width, ymax));
		return p;
	}
}